import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Write a description of class alligator here.
 * 
 * @author Brandon Jones 
 * @version (a version number or a date)
 */
public class Ted  extends Mover
{
    private int vSpeed = 0;
    private int vAccelerate = 2;
    private int maxSpeed = 5;
    private boolean down = true;
    private boolean right = true;
    private int frame = 0;
    
    private int cpx = 80;
    private int cpy = 280;
    private int cpl = 0;
    private int cpid = 0;
    private boolean cpd = true;
    
    public int deathCount = 0;
    
    public static final GreenfootSound JUMP_UP = new GreenfootSound("jumpup.wav");
    public static final GreenfootSound JUMP_DOWN = new GreenfootSound("jumpdown.wav");
    public static final GreenfootSound DEATH_NOISE = new GreenfootSound("death.wav");
    public static final GreenfootSound CHKPT_NOISE = new GreenfootSound("chkpt.wav");
    public Ted(){
        super();
        speed = 5;
    }
    /**
     * Act - do whatever the alligator wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        // Add your action code
       if(!((WheeWorld)getWorld()).gameOver){
        checkKeys();
        fall();
        checkReversal();
        checkEnemies();
        checkCheckpoint();
        changeDirection();
        checkEndWorld();
       }
    }
    
    public void checkCheckpoint(){
        Checkpoint chkpt = (Checkpoint)getOneIntersectingObject(Checkpoint.class);
        if(chkpt != null){
            if(chkpt.getID() != cpid){
                CHKPT_NOISE.play();
                WheeWorld world = (WheeWorld)getWorld();
                cpid = chkpt.getID();
                cpx = chkpt.getX();
                cpy = chkpt.getY();
                cpl = world.getLevel();
                cpd = chkpt.getDown();
                world.setCheckpointActive(cpid);
            }
        }
    }
    
    public void checkEnemies(){
        if(touches(Enemy.class)){
            deathCount++;
            DEATH_NOISE.play();
            GreenfootImage img = new GreenfootImage("chard.png");
            img.setTransparency(200);
            setImage(img);
            Greenfoot.delay(20);
            setLocation(cpx, cpy);
            down = cpd;
            WheeWorld world = (WheeWorld)getWorld();
            world.populate(cpl);
            img = new GreenfootImage("char0.png");
            setImage(img);
        }
    }
    
    public void checkEndWorld(){
        WheeWorld world = (WheeWorld)getWorld();
        if(getY() <=1){
            //setLocation(getX(), 399);
            world.changeLevel(0); //Exiting through the top;
        }
        if(getX() >= 599){
            //setLocation(1, getY());
            world.changeLevel(1); //Exiting through the right;
        }
        if(getY() >= 399){
            //setLocation(getX(), 1);
            world.changeLevel(2); //Exiting through the bottom;
        }
        if(getX() <= 0){
            //setLocation(599, getY());
            world.changeLevel(3); //Exiting through the left;
        }
    }
    
    public void checkReversal(){
       Actor reverse = getOneObjectAtOffset(0,0,Reversal.class);
       if(reverse != null){
            down = !down;
            if(!down){
                JUMP_UP.play();
            }
            else{
                JUMP_DOWN.play();
            }
        }
    }
    
    public boolean onGround(){
        Actor under;
        if(down){
            under = getOneObjectAtOffset(0, getImage().getHeight()/2, Ground.class);
            if(under != null){
                setLocation(getX(), under.getY() - (under.getImage().getHeight()/2) - + getImage().getHeight()/2); //Force self onto ground.
                if(under instanceof Destructible){
                    ((Destructible)under).setFade();
                }
                return true;
            }
        }
        else{
            under = getOneObjectAtOffset(0, -1 * getImage().getHeight()/2, Ground.class);
            if(under != null){
                setLocation(getX(), under.getY() + (under.getImage().getHeight()/2) + (getImage().getHeight()/2) -1); //Force self onto ground.
                if(under instanceof Destructible){
                    ((Destructible)under).setFade();
                }
                return true;
            }
        }
        return false; 
    }
    
    public boolean hitWall(){
        Actor front;
        if(right){
            front = getOneObjectAtOffset(getImage().getWidth()/2 + 2, 0, Ground.class);
        }
        else{
            front = getOneObjectAtOffset(-1 * getImage().getWidth()/2, 0, Ground.class);
        }
        return front !=null; 
    }
    
    private void checkKeys(){
        if(Greenfoot.isKeyDown("left")){
            moveLeft();
        }
        if(Greenfoot.isKeyDown("right")){
            moveRight();
        }
        if((Greenfoot.isKeyDown("space") || Greenfoot.isKeyDown("up")) && onGround()){
            vSpeed = 0;
            down = !down;
            if(!down){
                JUMP_UP.play();
            }
            else{
                JUMP_DOWN.play();
            }
            //vSpeed = 0;
        }
    }
    
    private void changeDirection(){
            if(!down){
                GreenfootImage img = getImage();
                setRotation(180);
            }
            else{
                setRotation(0);
            }
    }
    
    public void fall(){
        if(!onGround()){
            if(down){
             setLocation(getX(), getY() + vSpeed);
            }
            else{
             setLocation(getX(), getY() - vSpeed);
            }
            vSpeed += vAccelerate;
            if(vSpeed > maxSpeed){
                vSpeed = maxSpeed;
            }
        }
        else{
            vSpeed = 0;
        }
    }
    
    public GreenfootImage flip(GreenfootImage img){
        if(down && !right || !down && right){
            img.mirrorHorizontally();
        }
        return img;
    }
    
    public void moveRight(){
        right = true;
        if(!hitWall()){
            super.moveRight();
        }
        if(onGround()){
            String image = "char" + (frame % 8) + ".png";
            GreenfootImage img = new GreenfootImage(image);
            flip(img);
            setImage(img);
            frame++;
        }
        else{
            GreenfootImage img = new GreenfootImage("char0.png");
            setImage(img);
            frame = 0;
        }
    }
    
    public void moveLeft(){
        right = false;
        if(!hitWall()){
            super.moveLeft();
        }
        if(onGround()){
            String image = "char" + (frame % 8) + ".png";
            GreenfootImage img = new GreenfootImage(image);
            flip(img); 
            setImage(img);
            frame++;
        }
        else{
            GreenfootImage img = new GreenfootImage("char0.png");
            flip(img);
            setImage(img);
            frame = 0;
        }
    }
    
    public int getCPID(){
        return cpid;
    }
}
